#include "Plugin.h"

#include "resource.h"
#include <stdio.h>
#include <commctrl.h>

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


#include <commctrl.h>

typedef struct {
	const PLUGIN_FILELIST* pFileList; 
	int fileCount; 
	
	PLUGIN_BITMAP pBitmap; 

	const PLUGIN_PARAMS* pParams; 
} DATA; 

PLUGIN_API BOOL __stdcall GetPluginInfo( int version, PLUGIN_HEADER* header )
{
	if ( version >= PLUGIN_VERSION
		&& header->structSize >= sizeof(PLUGIN_HEADER) )
	{
		strcpy( header->label, "My Plugin" ); 
		header->capabilities = PLUGIN_CAP_ACCEPT_FILES | PLUGIN_CAP_FILTER; 
		return TRUE; 
	}
	return FALSE; 
}

static void Init( HWND hDlg, DATA* data )
{
	LV_COLUMN lvcol; 
	LV_ITEM lvi; 
	INT i; 
	HWND hWnd; 

	hWnd = GetDlgItem( hDlg, IDC_LIST ); 

	lvcol.mask = LVCF_TEXT; 
	lvcol.pszText = "Filename";
	lvcol.cx = 0; 
	lvcol.iSubItem = 0; 
	ListView_InsertColumn( hWnd, 0, &lvcol );  
	
	for ( i=0; i<data->fileCount; i++ )
	{
		lvi.mask = LVIF_TEXT;
		lvi.iItem = i;
		lvi.iSubItem = 0; 
		lvi.pszText = (char* )data->pFileList[i].filename; 
		ListView_InsertItem( hWnd, &lvi );
	}
	
	ListView_SetColumnWidth( hWnd, 0, LVSCW_AUTOSIZE );
	ListView_SetExtendedListViewStyle( hWnd, LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES ); 

	data->pBitmap = NULL; 

	char language[6]; 
	data->pParams->GetLanguage(language); 
	
	char string[256], ini_filename[MAX_PATH]; 
	GetModuleFileName( data->pParams->hInstance, ini_filename, sizeof(ini_filename) ); 
	char* ptr = strrchr( ini_filename, '\\' ); 
	if ( ptr )
		strcpy( ptr+1, "Plugin.lng" ); 
	GetPrivateProfileString( language, "1000", "", string, sizeof(string), ini_filename ); 
	if ( string[0] )
		SetDlgItemText( hDlg, IDC_ROTATE, string ); 
	GetPrivateProfileString( language, "1001", "", string, sizeof(string), ini_filename ); 
	if ( string[0] )
		SetDlgItemText( hDlg, IDC_SHARPEN, string ); 

	if ( data->pFileList == NULL )
		EnableWindow( hWnd, FALSE ); 
}

static void Load( HWND hDlg, INT index, DATA* data )
{
	char string[128]; 
	char* pText; 

	if ( index >= data->fileCount )
		return; 
	
	if ( data->pBitmap )
		data->pParams->FreeBitmap( data->pBitmap ); 
	data->pBitmap = NULL; 

	if (! data->pParams->LoadBitmap( data->pFileList[ index ].filename, &data->pBitmap ) )
		MessageBox( hDlg, data->pFileList[ index ].filename, "Problem", MB_OK ); 

	InvalidateRect( GetDlgItem( hDlg, IDC_BITMAP ), NULL, TRUE ); 

	string[0] = '\0'; 
	if ( data->pBitmap )
		sprintf( string, "%dx%d, %d", data->pParams->BitmapGetWidth(data->pBitmap), data->pParams->BitmapGetHeight(data->pBitmap), data->pParams->BitmapGetBits(data->pBitmap)); 

	SetDlgItemText( hDlg, IDC_TEXT1, string ); 

	pText = data->pParams->BitmapGetIPTC( data->pBitmap, 0x78 ); 
	SetDlgItemText( hDlg, IDC_TEXT2, pText ? pText : "" ); 
	data->pParams->Free( pText ); 
}

static void DrawItem( HWND hDlg, LPDRAWITEMSTRUCT pDIS, DATA* data )
{
	HBRUSH hBrush = CreateSolidBrush( RGB(255, 255, 255) ); 

	FillRect( pDIS->hDC, &pDIS->rcItem, hBrush ); 
	if ( data->pBitmap )
	{
		BITMAP bm; 
		HBITMAP hBitmap, hOldBitmap; 
		HDC hMemoryDC; 

		hBitmap = data->pParams->BitmapToDIB( data->pBitmap ); 
		if ( hBitmap )
		{
			INT width, height; 

			GetObject( hBitmap, sizeof(bm), &bm ); 
			
			hMemoryDC = CreateCompatibleDC( pDIS->hDC ); 
			hOldBitmap = (HBITMAP )SelectObject( hMemoryDC, hBitmap ); 
			SetStretchBltMode( pDIS->hDC, HALFTONE ); 
			
			width = pDIS->rcItem.right; 
			height = (pDIS->rcItem.right * bm.bmHeight) / bm.bmWidth; 
			if ( height > pDIS->rcItem.bottom )
			{
				height = pDIS->rcItem.bottom; 
				width = (pDIS->rcItem.bottom * bm.bmWidth) / bm.bmHeight; 
			}
			StretchBlt( pDIS->hDC, (pDIS->rcItem.right-width)/2, (pDIS->rcItem.bottom-height)/2, width, height, hMemoryDC, 0, 0, bm.bmWidth, bm.bmHeight, SRCCOPY ); 
			SelectObject( hMemoryDC, hBitmap ); 
			DeleteDC( hMemoryDC ); 

			DeleteObject( hBitmap ); 
		}
	}
	DeleteObject( hBrush ); 
}

static BOOL CALLBACK DialogDlgProc( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_INITDIALOG:
		SetWindowLong( hDlg, GWL_USERDATA, lParam ); 
		Init( hDlg, (DATA* )lParam ); 
		return TRUE; 
		break; 

	case WM_DESTROY:
		{
			DATA* data = (DATA* )GetWindowLong( hDlg, GWL_USERDATA ); 
			if ( data->pBitmap )
				data->pParams->FreeBitmap( data->pBitmap ); 
		}
		break; 

	case WM_DRAWITEM:
		{
			DATA* data = (DATA* )GetWindowLong( hDlg, GWL_USERDATA ); 
			DrawItem( hDlg, (LPDRAWITEMSTRUCT )lParam, data ); 
			return TRUE; 
		}
		break; 

	case WM_NOTIFY:
		switch ( ((LPNMHDR )lParam)->code )
		{
		case LVN_ITEMCHANGED:
			{
				LPNM_LISTVIEW lpNMListView = (LPNM_LISTVIEW )lParam; 
				DATA* data = (DATA* )GetWindowLong( hDlg, GWL_USERDATA ); 
				if ( lpNMListView->uChanged == LVIF_STATE
					&& ( lpNMListView->uNewState & LVIS_SELECTED ) )
					Load( hDlg, lpNMListView->iItem, data ); 
			}
			break; 
		}
		break; 

	case WM_COMMAND:
		{
			DATA* data = (DATA* )GetWindowLong( hDlg, GWL_USERDATA ); 
			switch(LOWORD(wParam))
			{
			case IDC_ROTATE:
				if ( data->pFileList == NULL )
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmap(); 
					data->pParams->Rotate( pBitmap, 90, TRUE ); 
					data->pParams->SetBitmap( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				else
				if ( data->pBitmap )
				{
					data->pParams->Rotate( data->pBitmap, 90, TRUE ); 
					InvalidateRect( GetDlgItem( hDlg, IDC_BITMAP ), NULL, TRUE ); 
				}
				break; 

			case IDC_GAMMA:
				if ( data->pFileList == NULL )
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmap(); 
					data->pParams->Adjust( pBitmap, 0, 0, 2.0 ); 
					data->pParams->SetBitmap( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				else
				if ( data->pBitmap )
				{
					data->pParams->Adjust( data->pBitmap, 0, 0, 2.0 ); 
					InvalidateRect( GetDlgItem( hDlg, IDC_BITMAP ), NULL, TRUE ); 
				}
				break; 

			case IDC_SHARPEN:
				if ( data->pFileList == NULL )
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmap(); 
					data->pParams->Sharpen( pBitmap, 80 ); 
					data->pParams->SetBitmap( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				else
				if ( data->pBitmap )
				{
					data->pParams->Sharpen( data->pBitmap, 80 ); 
					InvalidateRect( GetDlgItem( hDlg, IDC_BITMAP ), NULL, TRUE ); 
				}
				break; 

			case IDCANCEL:
				EndDialog( hDlg, TRUE ); 
			}
		}
		break; 
	}
	return FALSE; 
}

PLUGIN_API BOOL __stdcall Process( const PLUGIN_FILELIST* pFileList, int fileCount, const PLUGIN_PARAMS* pParams )
{
	InitCommonControls(); 

	DATA data; 
	data.pFileList = pFileList; 
	data.fileCount = fileCount; 
	data.pParams = pParams; 
	DialogBoxParam( pParams->hInstance, MAKEINTRESOURCE(IDD_DIALOG1), pParams->hWnd, DialogDlgProc, (LPARAM )&data ); 
	return TRUE; 
}

static BOOL CALLBACK DialogDlgProc2( HWND hDlg, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_INITDIALOG:
		SetWindowLong( hDlg, GWL_USERDATA, lParam ); 
		return TRUE; 
		break; 

	case WM_DESTROY:
		break; 

	case WM_COMMAND:
		{
			DATA* data = (DATA* )GetWindowLong( hDlg, GWL_USERDATA ); 
			switch(LOWORD(wParam))
			{
			case IDC_ROTATE90:
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmapSelection(); 
					data->pParams->Rotate( pBitmap, 90, TRUE ); 
					data->pParams->SetBitmapSelection( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				break; 

			case IDC_GAMMA:
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmapSelection(); 
					data->pParams->Adjust( pBitmap, 0, 0, 2.0 ); 
					data->pParams->SetBitmapSelection( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				break; 

			case IDC_SHARPEN:
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmapSelection(); 
					data->pParams->Sharpen( pBitmap, 80 ); 
					data->pParams->SetBitmapSelection( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				break; 

			case IDC_BLUR:
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmapSelection(); 
					data->pParams->Blur( pBitmap, 80 ); 
					data->pParams->SetBitmapSelection( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				break; 

			case IDC_LUT:
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmapSelection(); 
					unsigned char lut[256]; 
					for ( INT i=0; i<256; i++ )
						lut[i] = i * 2.0; 
					data->pParams->ApplyLut( pBitmap, lut ); 
					data->pParams->SetBitmapSelection( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				break; 

			case IDC_FILTER:
				{
					PLUGIN_BITMAP pBitmap = data->pParams->GetBitmapSelection(); 
					PLUGIN_FILTER filter; 
					filter.size = 3; 
					filter.bias = 128; 
					filter.divisor = 1; 
					filter.matrix[0] = -1; 
					filter.matrix[1] = -2; 
					filter.matrix[2] = 0; 
					filter.matrix[3] = -2; 
					filter.matrix[4] = 0; 
					filter.matrix[5] = 2; 
					filter.matrix[6] = 0; 
					filter.matrix[7] = 2; 
					filter.matrix[8] = 1; 
					data->pParams->Convolve( pBitmap, &filter ); 
					data->pParams->SetBitmapSelection( pBitmap ); 
					data->pParams->FreeBitmap( pBitmap ); 
				}
				break; 

			case IDCANCEL:
				EndDialog( hDlg, FALSE ); 
				break; 

			case IDOK:
				EndDialog( hDlg, TRUE ); 
			}
		}
		break; 
	}
	return FALSE; 
}

PLUGIN_API BOOL __stdcall ProcessFilter( const PLUGIN_PARAMS* pParams )
{
	InitCommonControls(); 

	DATA data; 
	memset( &data, 0, sizeof(data) ); 
	data.pParams = pParams; 
	return DialogBoxParam( pParams->hInstance, MAKEINTRESOURCE(IDD_DIALOG2), pParams->hWnd, DialogDlgProc2, (LPARAM )&data ); 
}
