/*
 *                        XPlugins.h
 *
 *  XnView Copyright (c) 1998-2006 Pierre-e Gougelet
 *  All rights reserved
 *
 *
 */

#ifndef __XPLUGINS_H__
#define __XPLUGINS_H__

#ifdef __cplusplus
extern "C" {
#endif

#define PLUGIN_VERSION 0x0200

typedef struct {
		unsigned char red, green, blue, alpha; 
	} PLUGIN_COLOR; 

typedef struct {
		char filename[MAX_PATH]; 
	} PLUGIN_FILELIST; 

typedef struct {
		int removeMetadata; 
		int jpegQuality; 
		int pngLevel; 
	} PLUGIN_SAVE_PARAMS; 

typedef struct {
		int x, y, w, h; 
	} PLUGIN_RECT; 

typedef struct {
		int size;  // Max 7
    int matrix[7*7]; 
    int divisor; 
    int bias; 
  } PLUGIN_FILTER; 

typedef void* PLUGIN_BITMAP; 
typedef void* PLUGIN_METADATA; 

typedef enum {
		PLUGIN_BITMAP_BINARY=0, 
		PLUGIN_BITMAP_COLORS, 
		PLUGIN_BITMAP_GREYSCALE, 
		PLUGIN_BITMAP_RGB, 
		PLUGIN_BITMAP_RGBA, 
	} PLUGIN_BITMAP_TYPE; 

/* Format name available:
		jpeg, bmp, png
 */

#define PLUGIN_RESIZE_ORIENTATION 0x01
#define PLUGIN_RESIZE_KEEPRATIO		0x02

typedef struct {
		int structSize; 
		
		HINSTANCE hInstance; 
		HWND hWnd; 
		
		// --
		
		void (__stdcall * GetLanguage)(char language[8]); 
		void (__stdcall * GetPathnameForINI)(char* pathname, int filename_length); 
/*		BOOL (__stdcall * GetFilename)(char* filename, int filename_length, BOOL for_saving); */
		void (__stdcall * GetXnViewPathname)(char* pathname, int filename_length); 
		void (__stdcall * GetXnViewBasePathname)(char* pathname, int filename_length); 
		void (__stdcall * GetXnViewAddonPathname)(char* pathname, int filename_length); 
		BOOL (__stdcall * GetFolder)(HWND hDlg, char* filename, int filename_length); 
		BOOL (__stdcall * GetOutputFilename)(HWND hDlg, const char* base_pathname, const char* extension, char* filename, int filename_length); 
		
		// --

		HBITMAP (__stdcall * GetThumbnail)(const char* filename, int max_width, int max_height); 
		void (__stdcall * FreeThumbnail)(HBITMAP hBitmap); 
		HBITMAP (__stdcall * LoadDIB)(const char* filename); 
		HBITMAP (__stdcall * LoadDIBEx)(const char* filename, PLUGIN_METADATA* metadata, char* format_name, int format_name_length); 
		HBITMAP (__stdcall * BitmapToDIB)(const PLUGIN_BITMAP bitmap); 
		BOOL (__stdcall * SaveDIB)(const char* filename, const char* format_name, const HBITMAP hBitmap, const PLUGIN_SAVE_PARAMS* save_params); 
		BOOL (__stdcall * SaveDIBEx)(const char* filename, const char* format_name, const HBITMAP hBitmap, const PLUGIN_METADATA metadata, const PLUGIN_SAVE_PARAMS* save_params); 
		void (__stdcall * FreeMetadata)(PLUGIN_METADATA metadata); 
		
		BOOL (__stdcall * GetBitmapInfo)(const char* filename, int* width, int* height, char* format_name, int format_name_length); 
		BOOL (__stdcall * LoadBitmap)(const char* filename, PLUGIN_BITMAP* bitmap); 
		BOOL (__stdcall * LoadBitmapEx)(const char* filename, PLUGIN_BITMAP* bitmap, char* format_name, int format_name_length); 
		BOOL (__stdcall * SaveBitmap)(const char* filename, const char* format_name, const PLUGIN_BITMAP bitmap, const PLUGIN_SAVE_PARAMS* save_params); 
		void (__stdcall * FreeBitmap)(PLUGIN_BITMAP bitmap); 
		
		PLUGIN_BITMAP_TYPE (__stdcall * BitmapGetType)(PLUGIN_BITMAP bitmap); 
		INT (__stdcall * BitmapGetWidth)(PLUGIN_BITMAP bitmap); 
		INT (__stdcall * BitmapGetHeight)(PLUGIN_BITMAP bitmap); 
		INT (__stdcall * BitmapGetBits)(PLUGIN_BITMAP bitmap); 
		INT (__stdcall * BitmapGetBytesPerLine)(PLUGIN_BITMAP bitmap); 
		
		// -- Only for filter plugin...
		void* (__stdcall * BitmapGetBuffer)(PLUGIN_BITMAP bitmap, int y); // RGB/RGBA
		PLUGIN_BITMAP (__stdcall * BitmapNew)(int width, int height, PLUGIN_BITMAP_TYPE type); 
		void (__stdcall * GetBitmapFilename)(char* pathname, int filename_length); 
		BOOL (__stdcall * BitmapCopy)(PLUGIN_BITMAP bitmap, const PLUGIN_BITMAP org_bitmap); 
		PLUGIN_BITMAP (__stdcall * GetBitmap)( void ); 
		PLUGIN_BITMAP (__stdcall * GetBitmapSelection)( void ); 
		BOOL (__stdcall * GetSelection)(PLUGIN_RECT* rect); 
		BOOL (__stdcall * SetBitmap)(PLUGIN_BITMAP bitmap); 
		BOOL (__stdcall * SetBitmapSelection)(PLUGIN_BITMAP bitmap); 
		HBITMAP (__stdcall * GetDIB)(void); 
		HBITMAP (__stdcall * GetDIBSelection)(void); 
		BOOL (__stdcall * SetDIB)(HBITMAP hBitmap); 
		BOOL (__stdcall * SetDIBSelection)(HBITMAP hBitmap); 
		
		// --
		
		char* (__stdcall * BitmapGetIPTC)(PLUGIN_BITMAP bitmap, UINT id); 
		char* (__stdcall * BitmapGetEXIF)(PLUGIN_BITMAP bitmap, UINT tag); 
		void (__stdcall * Free)(void* ptr); 
		
		BOOL (__stdcall * Resize)(PLUGIN_BITMAP bitmap, int width, int height, UINT flag); 
		BOOL (__stdcall * CanvasResize)(PLUGIN_BITMAP bitmap, int width, int height, const PLUGIN_COLOR* color); 
		BOOL (__stdcall * Rotate)(PLUGIN_BITMAP bitmap, int degrees, int smooth); 
		BOOL (__stdcall * FlipVertical)(PLUGIN_BITMAP bitmap); 
		BOOL (__stdcall * FlipHorizontal)(PLUGIN_BITMAP bitmap); 
		BOOL (__stdcall * Adjust)(PLUGIN_BITMAP bitmap, int brightness, int contrast, double gamma); 
		BOOL (__stdcall * AdjustHLS)(PLUGIN_BITMAP bitmap, int hue, int lightness, int saturation); 
		BOOL (__stdcall * ApplyLut)(PLUGIN_BITMAP bitmap, unsigned char lut[256]); 
		BOOL (__stdcall * Convolve)(PLUGIN_BITMAP bitmap, const PLUGIN_FILTER* pFilter); 
		BOOL (__stdcall * Blur)(PLUGIN_BITMAP bitmap, int percent); 
		BOOL (__stdcall * Sharpen)(PLUGIN_BITMAP bitmap, int percent); 
		BOOL (__stdcall * Crop)(PLUGIN_BITMAP bitmap, int x, int y, int w, int h); 
		BOOL (__stdcall * ConvertInRGB)(PLUGIN_BITMAP bitmap); 
		BOOL (__stdcall * ConvertInColors)(PLUGIN_BITMAP bitmap); 
		
	} PLUGIN_PARAMS; 

#define PLUGIN_CAP_NORMAL						0x0000
#define PLUGIN_CAP_ACCEPT_FILES			0x0001
#define PLUGIN_CAP_ACCEPT_FOLDERS		0x0002
#define PLUGIN_CAP_FILTER						0x0004
#define PLUGIN_CAP_SLIDESHOW				0x0008

#define PLUGIN_HEADER_STRING_LENGTH	256

typedef struct {
		int structSize; 

		char label[PLUGIN_HEADER_STRING_LENGTH]; 
		char author[PLUGIN_HEADER_STRING_LENGTH]; 
		char version[PLUGIN_HEADER_STRING_LENGTH]; 
		char comment[PLUGIN_HEADER_STRING_LENGTH]; 

		UINT capabilities; 
		HICON hIcon; 
	} PLUGIN_HEADER; 

typedef struct {
		int structSize; 
		
		const char* soundFilename; 
		INT delay; 
	} PLUGIN_SLIDE_PARAMS; 

typedef BOOL (__stdcall *_GetPluginInfo)( int version, PLUGIN_HEADER* header ); 
typedef BOOL (__stdcall *_Process)( const PLUGIN_FILELIST* pFileList, int fileCount, const PLUGIN_PARAMS* params ); 
typedef BOOL (__stdcall *_ProcessSlide)( const PLUGIN_FILELIST* pFileList, int fileCount, const PLUGIN_SLIDE_PARAMS* slide_params, const PLUGIN_PARAMS* params ); 
typedef BOOL (__stdcall *_ProcessFilter)( const PLUGIN_PARAMS* params ); 

#ifdef __cplusplus
}; 
#endif

#endif

